# -*- coding: utf-8 -*-
"""
Базовые классы для queue backends
"""
from __future__ import absolute_import
import abc
import six


class Queue(six.with_metaclass(abc.ABCMeta, object)):
    """
    Сервер очереди
    """

    def __init__(self, addr, quiet=False):
        """
        :addr - (host, port), на котором сервер создаст сокет
        """
        self._addr = addr
        self._quiet = quiet

    @abc.abstractmethod
    def run(self):
        pass


class _Client(object):
    """
    Абстракстный клиент
    """

    def __init__(self, ident, queue_addr, queue_access=None, quiet=False):
        """
        :ident      - идентификатор отправителя
        :queue_addr - (host, port) сервера очереди
        :queue_access - (login, password, vhost) для доступа к брокеру
        """
        self._ident = ident
        self._queue_addr = queue_addr
        self._queue_access = queue_access
        self._quiet = quiet


class Sender(six.with_metaclass(abc.ABCMeta, _Client)):
    """
    Клиент-отправитель
    """

    @abc.abstractmethod
    def send(self, data, uid=None, topic=None):
        """
        Отправляет сообщение получателю посредством очереди
        :uid :: int - id socket-соединения
        :topic - тема сообщения
        :data - данные для отправки
        """
        pass


class Receiver(six.with_metaclass(abc.ABCMeta, _Client)):
    """
    Клиент-получатель
    """

    @staticmethod
    def _default_callback(uid, topic, data):
        """
        Умолчательный обработчик получения сообщений
        """
        pass

    # атрибут, хранящий функцию - обработчик получения сообщений
    callback = _default_callback

    @abc.abstractmethod
    def process(self):
        """
        Производит получение сообщений
        (это callback для асинхронного ядра tornado)
        """
        pass
