# -*- coding: utf-8 -*-
"""
Подсистема наcтройки компонент через ком.строку
"""
from __future__ import print_function
from __future__ import absolute_import
import optparse
import sys

PORT = {
    'switches': ("-p", "--port"),
    'dest': "port",
    'type': "int",
    'help': "The TCP port on which the process listens for client connections. "
            "Default 9999 for pushme.server and 4000 for pushme.queue"
}

HOST = {
    'switches': ("-H", "--host"),
    'dest': "host",
    'type': "str",
    'help': "The hostname or IP addresses on which process should listen "
            "for client connections. Default: all IPs"
}

QUIET = {
    'switches': ("-q", "--quiet"),
    'dest': "quiet",
    'action': "store_true",
    'default': False,
    'help': "be quiet"
}

DEBUG = {
    'switches': ("-d", "--debug"),
    'dest': "debug",
    'action': "store_true",
    'default': False,
    'help': "be debug"
}

QUEUE_HOST = {
    'switches': ("-m", "--mq-host"),
    'dest': "queue_host",
    'type': "str",
    'default': "localhost",
    'help': "Hostname or ip for message queue server. Default: localhost"
}

QUEUE_PORT = {
    'switches': ("-P", "--mq-port"),
    'dest': "queue_port",
    'type': "int",
    'help': "TCP port for message queue server."
            "Default: 4000  for snakemq bacckend, 5672 for rabbitmq backend"
}

QUEUE_BACKEND = {
    'switches': ("-B", "--mq-backend"),
    'dest': "queue_backend",
    'type': "choice",
    'help': "Message queue backend. Default: snakemq",
    'choices': ("snakemq", "rabbitmq"),
}

QUEUE_USER = {
    'switches': ("--mq-user",),
    'dest': "queue_user",
    'type': "str",
    'help': "Message queue user"
}

QUEUE_PASSWORD = {
    'switches': ("--mq-password",),
    'dest': "queue_password",
    'type': "str",
    'help': "Message queue password"
}

QUEUE_VHOST = {
    'switches': ("--mq-vhost",),
    'dest': "queue_vhost",
    'type': "str",
    'help': "Message queue vhost"
}

LISTENER_OPTIONS = (HOST, PORT)
MQ_OPTIONS = (
    QUEUE_BACKEND, QUEUE_HOST, QUEUE_PORT, QUEUE_USER, QUEUE_PASSWORD,
    QUEUE_VHOST)
ALL_OPTIONS = LISTENER_OPTIONS + MQ_OPTIONS + (QUIET, DEBUG)


def configure(prog, options, **defaults):
    """
    Возвращает парсер ключей ком.строки для компоненты
    """
    option_list = []
    for o in options:
        o = o.copy()
        try:
            o['default'] = defaults[o['dest']]
        except KeyError:
            pass
        option_list.append(
            optparse.make_option(*o.pop('switches'), **o))

    parser = optparse.OptionParser(
        option_list=option_list, prog=prog)

    config, unparsed = parser.parse_args()
    if unparsed:
        print("Unknown arg(s)!:\n" + "\n".join(unparsed))
        parser.print_help()
        sys.exit(1)
    return config
