// клиентская сторона socket-соединения

var pushMeConnection = {

    conn: undefined,

    reconnectTimeout: 5000,

    listenerIdSeq: 1,

    listeners: {},
    callbacks: {},

    MAGIC_TOPIC: '-<*>-', // id топика, на который "подписываются"
                          // слушатели, для которых топик не указан.
                          // Такие слушетели будут получать все сообщения.

    // создает SockJS-соединение с сервером (Tornado) с параметрами:
    // * host - протокол и хост без завершающего символа '/' в конце.
    //     Например: 'http://localhost' или 'https://192.168.1.1'
    // * port - необязательный параметр порта. По умолчанию для
    //     http - 80, а для https - 443.
    // * uid - id для идентификации соединения на серверной стороне.
    //     Например: id текущего пользователя
    // * url_suffix - необязательный параметр url-суффикса, который
    //     добавляется после port-порта, но перед '/pull'. Используется,
    //     например, в случаях когда на одном хосте и порте запущено
    //     несколько socket-сервисов (то есть используется несколько
    //     инстансов-стендов на одном хосте).
    //     Например: если этот параметр указывается как '/kaliningrad',
    //     то полный url для SockJS получится типа:
    //         'http://192.168.1.1:80/kaliningrad/pull'
    connect: function(host, port, uid, url_suffix) {
        var pushme = this;
        if (!this.conn) {
            var sockjs_url = host;
            if (port) {
                sockjs_url += ':' + port;
            };
            if (url_suffix) {
                sockjs_url += url_suffix;
            };
            sockjs_url += '/pull';
            var conn = new SockJS(sockjs_url);

            conn.onopen = function() { conn.send(uid); };

            conn.onmessage = function(msg) {
                pushme.processMessage(msg);
            };

            conn.onclose = function() {
                delete pushme.conn;
                var tOut = setTimeout(
                    function() {
                        clearTimeout(tOut);
                        pushme.connect(host, port, uid, url_suffix);
                    },
                    pushme.reconnectTimeout
                );
            };

            this.conn = conn;

        } else {
            console.error('Already connected!');
        };
    },

    processMessage: function(e) {
        var msg = JSON.parse(e.data),
            topic = msg.topic,
            fanouts = this.callbacks[this.MAGIC_TOPIC],
            handlers = this.callbacks[topic];

        for (var key in fanouts) {
            fanouts[key](msg.data, topic);
        };

        for (var key in handlers) {
            handlers[key](msg.data, topic);
        };
    },

    subscribe: function(callback, topic) {
        // добавляет callback к подписчикам, возвращает id,
        // с помощью которого callback может быть отписан
        // если topic не указан - подписчик будет получать все
        this.listenerIdSeq += 1;
        var id = this.listenerIdSeq,
            topic = topic || this.MAGIC_TOPIC;
        if (!this.callbacks[topic]) {
            this.callbacks[topic] = {}
        };
        this.callbacks[topic][id] = callback;
        this.listeners[id] = topic;
        return id;
    },

    unsubscribe: function(id) {
        // отписывает callback по идентификатору id,
        // который ранее вернул метод subscribe
        if (this.listeners[id]) {
            delete this.callbacks[this.listeners[id]][id];
            delete this.listeners[id];
        } else {
            console.error('Wrong listener id:', id);
        };
    },
};
